@@ -131,9 +131,9 @@ type chunkedIndexReader struct {
 
 func newChunkedIndexReader(ctx context.Context, bkt objstore.BucketReader, id ulid.ULID) (*chunkedIndexReader, int, error) {
 	indexFilepath := filepath.Join(id.String(), block.IndexFilename)
-	size, err := bkt.ObjectSize(ctx, indexFilepath)
+	attrs, err := bkt.Attributes(ctx, indexFilepath)
 	if err != nil {
-		return nil, 0, errors.Wrapf(err, "get object size of %s", indexFilepath)
+		return nil, 0, errors.Wrapf(err, "get object attributes of %s", indexFilepath)
 	}
 
 	rc, err := bkt.GetRange(ctx, indexFilepath, 0, index.HeaderLen)
@@ -164,7 +164,7 @@ func newChunkedIndexReader(ctx context.Context, bkt objstore.BucketReader, id ul
 	ir := &chunkedIndexReader{
 		ctx:  ctx,
 		path: indexFilepath,
-		size: size,
+		size: uint64(attrs.Size),
 		bkt:  bkt,
 	}
 
@@ -637,8 +637,8 @@ func newBinaryTOCFromByteSlice(bs index.ByteSlice) (*BinaryTOC, error) {
 	}, nil
 }
 
-func (r BinaryReader) IndexVersion() int {
-	return r.indexVersion
+func (r BinaryReader) IndexVersion() (int, error) {
+	return r.indexVersion, nil
 }
 
 // TODO(bwplotka): Get advantage of multi value offset fetch.
@@ -871,7 +871,7 @@ func yoloString(b []byte) string {
 	return *((*string)(unsafe.Pointer(&b)))
 }
 
-func (r BinaryReader) LabelNames() []string {
+func (r BinaryReader) LabelNames() ([]string, error) {
 	allPostingsKeyName, _ := index.AllPostingsKey()
 	labelNames := make([]string, 0, len(r.postings))
 	for name := range r.postings {
@@ -882,7 +882,21 @@ func (r BinaryReader) LabelNames() []string {
 		labelNames = append(labelNames, name)
 	}
 	sort.Strings(labelNames)
-	return labelNames
+	return labelNames, nil
 }
 
 func (r *BinaryReader) Close() error { return r.c.Close() }
+
+type realByteSlice []byte
+
+func (b realByteSlice) Len() int {
+	return len(b)
+}
+
+func (b realByteSlice) Range(start, end int) []byte {
+	return b[start:end]
+}
+
+func (b realByteSlice) Sub(start, end int) index.ByteSlice {
+	return b[start:end]
+}
